@@ -66,7 +66,16 @@ class Query:
 class Client:
 
     def __init__(self, url: str, username: str, pwd: str, *args, **kwargs):
-        """ Create a conection with nettime app using recived parameters. """
+        """
+        Create a conection with nettime app using recived parameters.
+
+        :param url: Nettime url. Eg https://server-name:8091/.
+        :param username: Nettime username.
+        :param pwd: Nettime password.
+
+        :return: :class:`Client` object
+        :rtype: Client
+        """
 
         super().__init__(*args, **kwargs)
         self.client_url = urlparse(url)
@@ -109,7 +118,7 @@ class Client:
         return bool(self.headers) and bool(self.access_token)
 
     def connect(self):
-        """ Connect the client to get access_token and headers values. """
+        """ Connect the client to set access_token and headers values. """
 
         if self.is_connected:
             return
@@ -152,8 +161,8 @@ class Client:
         :param params: (optional) Dictionary, list of tuples or bytes to send
             in the query string for the :class:`Request`.
         :param \*\*kwargs: Optional arguments that ``request`` takes.
-        :return: :class:`dict` object
-        :rtype: dict
+        :return: json object
+        :rtype: json
         """
 
         if not self.is_connected:
@@ -165,6 +174,7 @@ class Client:
             "params": params,
             "headers": self.headers,
             "timeout": kwargs.get("timeout", 10),
+            "stream": kwargs.get("stream", False)
         }
 
         # consulting nettime
@@ -178,6 +188,10 @@ class Client:
         # raise if was an error
         if response.status_code not in range(200, 300):
             raise ConnectionError(response.text)
+        
+        # if request is stream type, return all response
+        if kwargs.get("stream"):
+            return response
 
         # to json
         return response.json()
@@ -192,8 +206,8 @@ class Client:
         :param json: (optional) json data to send in the body of the 
             :class:`Request`.
         :param \*\*kwargs: Optional arguments that ``request`` takes.
-        :return: :class:`dict` object
-        :rtype: dict
+        :return: json object
+        :rtype: json
         """
 
         # wait active conection
@@ -206,7 +220,7 @@ class Client:
             "data": data,
             "json": json,
             "headers": self.headers,
-            "timeout": kwargs.get("timeout", 10),
+            "timeout": kwargs.get("timeout", 10)
         }
 
         # consulting nettime
@@ -222,7 +236,15 @@ class Client:
             raise ConnectionError(response.text)
 
         # to json -> json
-        return response.json()
+        json_response = response.json()
+
+        # get task results if is generated
+        if isinstance(json_response, dict) and \
+                json_response.get('taskId', None):
+            json_response = self.get_task_response(json_response.get('taskId'))
+
+        # return json response
+        return json_response
 
     def get_headers(self):
         """ Return headers for a specific conection """
@@ -238,7 +260,7 @@ class Client:
         }
 
     def disconnect(self):
-        """ Disconnect a client to lock the access_token. """
+        """ Disconnect a client to clean the access_token. """
 
         if not self.is_connected:
             return
@@ -262,8 +284,13 @@ class Client:
 
     def get_days_offset(self, days: list):
         """ 
-        Convert a list of datetime.date or str format to int offset with \
+        Convert a list of datetime.date or str format to int offset with 
         self.setting.firstDate.
+
+        :param days: list of datetime.date or formatted str to get int offsets.
+        
+        :return: :class:`list` object
+        :rtype: list
         """
 
         # wait active conection
@@ -289,10 +316,15 @@ class Client:
 
         return days_numbers
 
-    def get_days_from_ofsets(self, ofsets: list):
+    def get_days_from_offsets(self, offsets: list):
         """ 
         Convert a list of int offset to datetime.date or str format with \
         self.setting.firstDate.
+
+        :param offsets: list of int to get dates.
+        
+        :return: :class:`datetime.date` object
+        :rtype: datetime.date
         """
 
         # wait active conection
@@ -313,9 +345,32 @@ class Client:
             dates.append(date.isoformat())
         
         return dates
+
+    def get_app_resource(self, name: str, **kwargs):
+        """
+        Get a resource from 'AppResources' folder. You can set export path in
+        nettime folder and after use this method to get export content.
+
+        :param name: str with name of resource -including extension-.
+        :param **kwargs: Optional arguments that ``request`` takes.
+
+        :return: :class:`dict` object or stream if 'stream' in kwargs
+        :rtype: json or stream
+        """
+
+        # request.get
+        return self.get(path=f'/AppResources/{name}', **kwargs)
         
     def get_fields(self, container: str, filterFields: bool = False):
-        """ Get all fields of an specific container. """
+        """
+        Get all fields of an specific container.
+        
+        :param container: (str) Name of nettime container.
+        :param filterFields: (bool) True if needs ignore expression fields.
+        
+        :return: json object
+        :rtype: json
+        """
 
         # prepare task parameters
         params = {
@@ -328,7 +383,18 @@ class Client:
     
     def get_elements(self, container: str, query = Query(["id", "name"]), \
             *args, **kwargs):
-        """ Get elements of an specific container for general propose. """
+        """
+        Get elements of an specific container for general propose.
+        
+        :param container: (str) Name of nettime container.
+        :param query: (Query) Fields you want to get. Must be an instanceb of
+            nettime6.Query. See more information in the class documentation.
+
+        :param \*\*kwargs: Optional arguments that ``request`` takes.
+
+        :return: json object
+        :rtype: json
+        """
 
         # prepare task parameters
         params = {
@@ -351,7 +417,17 @@ class Client:
         return json_response
 
     def get_employees(self, query=Query(["id", "nif"]), *args, **kwargs):
-        """ Get employees from nettime. """
+        """
+        Get employees from nettime. 
+        
+        :param query: (Query) Fields you want to get. Must be an instanceb of
+            nettime6.Query. See more information in the class documentation.
+        :param \*args: Optional arguments that ``request`` takes.
+        :param \*\*kwargs: Optional arguments that ``request`` takes.
+
+        :return: json object
+        :rtype: json
+        """
 
         # use get general propose
         employees = self.get_elements(
@@ -737,7 +813,7 @@ class Client:
             fields or results. The order of the values does matter.
         :param dateIni: Start day where you want to calculate.
         :param dateIni: End day where you want to calculate.
-        :param interfilters: (Optional) If you use the dimension "system", 
+        :param interfilters: (Optional) If you use the "system" dimension, 
             specify the ids of the results in this parameter.
         :param filters: (Optional) Nettime compatible filter expression.
         :param ids: (Optional) List of employee ids in case you want to filter.
@@ -884,9 +960,82 @@ class Client:
 
         return self.save_element(**data)
 
+    def get_timetypes_ids(self):
+        """
+        Gets and returns a list of ids of timetypes with a nettime client.
+        """
 
+        # get nt resposne
+        nt_timetypes = self.get_elements("Incidencia").get('items')
 
-        
+        # parse response to list
+        timetypes = []
+        for timetype in nt_timetypes:
+            timetypes.append({"id": timetype.get("id")})
 
+        return timetypes
 
+    def get_readers_ids(self):
+        """
+        Gets and returns a list of ids of readers with a nettime client.
+        """
+
+        # get nt resposne
+        nt_readers = self.get_elements("Lector").get('items')
+
+        # parse response to list
+        readers = []
+        for reader in nt_readers:
+            readers.append({"id": reader.get("id")})
+
+        return readers
+
+    def import_employee(self, structure: dict, **kwargs):
+        """
+        Create an employe from a structure. Update if exists, create if not.
+        """
+
+        # employee structure
+        data = {"container": "Persona"}
+
+        # search employee by nif
+        query = Query(
+            fields=["id", "nif"],
+            filterExp=f'this.nif = "{structure.get("nif")}"',
+        )
+        results = self.get_employees(query=query)
+
+        # safety only
+        if results.get('total') > 1:
+            raise ValueError("Más de un empleado con el mismo DNI.")
+
+        # update employee
+        if results.get('total') == 1:
+            # set element
+            data["elements"] = [results.get('items')[0].get('id')]
+            # empty data
+            dataObj = {}
+
+        # create element
+        else:
+            # create form and assign all timetypes and readers
+            dataObj = self.get_create_form(container="Persona")
+
+            # assign all if not in structure
+            if not structure.get('TimeTypesEmployee'):
+                dataObj.update({"TimeTypesEmployee": self.get_timetypes_ids()})
+            
+            # assign all if not in structure
+            if not structure.get('Readers'):
+                dataObj.update({"Readers": self.get_readers_ids()})
+            
+            # delete elements kw
+            if data.get("elements", None):
+                del data["elements"]
+
+        dataObj.update(structure)
+        data["dataObj"] = dataObj
+
+        # save employee
+        return self.save_element(**data)
 
