@@ -5,8 +5,7 @@ from base64 import b64encode, b64decode
 import requests
 import datetime
 import re
-
-#import nettime6 as nt6
+import math
 
 class JsonObject:
     """ Convert a dict['element'] for access like object.property. """
@@ -31,7 +30,7 @@ class Client:
     class Authentication:
         def __init__(self, **kwargs):
             self.access_token = kwargs.get('access_token')
-            self.token_type = kwargs.get('token_type', '').capitalize()
+            self.token_type = kwargs.get('token_type', 'Bearer').capitalize()
             self.expires = self.get_expires(kwargs.get('expires_in'))
 
             self.rol = kwargs.get('rol')
@@ -134,7 +133,7 @@ class Client:
 
     def get(self, path: str, params: dict = None, **kwargs):
         """
-        Sends a GET request to nettime url.
+        Sends a GET request to visma url.
 
         :param path: path to add to URL for the new :class:`Request` object.
         :param params: (optional) Dictionary, list of tuples or bytes to send
@@ -157,7 +156,8 @@ class Client:
             "url": urljoin(self.client_url.geturl(), path),
             "params": params,
             "headers": self.headers,
-            "timeout": kwargs.get("timeout", 10),
+            "timeout": kwargs.get("timeout", 20),
+            "stream": kwargs.get("stream", False)
         }
 
         # consulting nettime
@@ -169,13 +169,18 @@ class Client:
 
         # to json -> json
         try:
+            # if request is stream type, return all response
+            if kwargs.get("stream"):
+                return response
+
+            # return json response
             return response.json()
         except:
             return {}
 
     def post(self, path, data=None, json=None, **kwargs):
         """
-        Sends a POST request to nettime url.
+        Sends a POST request to visma url.
 
         :param url: URL for the new :class:`Request` object.
         :param data: (optional) Dictionary, list of tuples, bytes, or file-like
@@ -201,7 +206,7 @@ class Client:
             "data": data,
             "json": json,
             "headers": self.headers,
-            "timeout": kwargs.get("timeout", 10),
+            "timeout": kwargs.get("timeout", 20),
         }
 
         # consulting nettime
@@ -257,7 +262,7 @@ class Client:
         self.connect()
 
     def get_employees(self, employee: str = None, extension: str = None, \
-            **kwargs):
+            all_pages: bool = False, **kwargs):
         """
         Use the endpoint to obtain the employees with the received data.
         
@@ -293,13 +298,29 @@ class Client:
         params = {
             "orderBy": kwargs.get("orderBy", None),
             "page": kwargs.get("page", None),
-            "pageSize": kwargs.get("pageSize", 5),
+            "pageSize": kwargs.get("pageSize", 50),
             "active": kwargs.get("active", None),
             "updatedFrom": kwargs.get("updatedFrom", None)
         }
 
         # request.get -> json
-        return self.get(path=path, params=params)
+        response = self.get(path=path, params=params)
+        
+        # recursive call to get all pages values
+        if all_pages and response.get('totalCount', 0) > params.get('pageSize'):
+            # calculate num of pages
+            num_pages = math.ceil(
+                response.get('totalCount') / params.get('pageSize')
+            )
+            # recursive get and extend response values
+            for i in range(2, num_pages + 1):
+                # update page
+                params['page'] = i
+                response['values'].extend(
+                    self.get(path=path, params=params).get('values')
+                )
+        # return elements
+        return response
 
     def get_addresses(self, address: str = None, extension: str = None, \
             **kwargs):
@@ -487,8 +508,60 @@ class Client:
         # request.get -> json
         return self.get(path=path, params=params)
 
-    def post_pay_elements(self, **kwargs):
-        pass
+    def post_pay_elements(self, values: list, **kwargs):
+        """
+        Use the endpoint to post recived payment/s data.
+
+        :param values: list of dict for send to api. Each dict must be like:
+            {
+                "employeeExternalId": "string",         
+                # Legajo del empleado (Obligatorio) 
+                "periodFrom": "string",                 
+                # Descripcion Periodo (opcional) para retroactividad
+                "periodTo": "string",                   
+                #  Descripcion Periodo  (opcional) para retroactividad
+                "reason": "string",                     
+                # razón (descriptivo)   (opcional)
+                "reasonTypeExternalId": "string",       
+                # id tipo razon (opcional)
+                "action": 0,                            
+                # 0 inserta, 1 Actualiza 
+                "retroactive": Boolean,                 
+                # true o false (retroactividad)
+                "journalModelId": 0,                    
+                # id modelo de asiento  (opcional)
+                "journalModelStructureId1": 0,          
+                # id estructura 1 del modelo de asiento  (opcional)
+                "journalModelStructureId2": 0,          
+                # id estructura 2 del modelo de asiento  (opcional)
+                "journalModelStructureId3": 0,          
+                # id estructura 3 del modelo de asiento  (opcional)
+                "conceptExternalId": "string",          
+                # Codigo externo del concepto (Obligatorio)
+                "parameterId": 0,                       
+                # código interno del del parámetro (Obligatorio)
+                "dateFrom": "2020-10-01T18:23:50.691Z", 
+                # fecha de vigencia desde (opcional) 
+                "dateTo": "2020-10-01T18:23:50.691Z",   
+                # fecha de vigencia hasta  (opcional)
+                "value": 0                              
+                # valor de la novedad (Obligatorio)
+            }
+        
+        :return: json object
+        :rtype: json
+        """
+
+        # path prepare
+        path = '/WebApi/pay-elements/individual'
+
+        # json prepare
+        _json = {
+            "values": values
+        }
+
+        # request.get -> json
+        return self.post(path=path, json=_json, **kwargs)
 
     def get_payments(self, extension: str, **kwargs):
 
@@ -505,7 +578,7 @@ class Client:
         # request.get -> json
         return self.get(path=path, params=params)
 
-    def get_payments(self, extension: str, **kwargs):
+    def get_payrolls(self, extension: str, **kwargs):
 
         # path prepare
         path = f'/WebApi/payrolls/{extension}'
@@ -705,3 +778,4 @@ class Client:
 
         # request.get -> json
         return self.get(path='/WebApi/version')
+
